# MIT License

# Copyright (c) 2021 Jorge Marín

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# Contributors:
#     Jorge Marín - initial Package version
"""
objects module
"""
from datetime import datetime

class Price:
    """
    This is a class for instantiate a Price object.

    Attributes:
        __raw_hour (str): The session interval.
        __date (str): The session date.
        __pen_price (str): The price for Iberian Peninsula and Balearic Islands.
        __can_price (str): The price for Canary Island and Melilla.

    """
    def __init__(self, raw_hour:str, date:str, pen_price:str, can_price:str):
        """
        The constructor for Price class.

        Parameters:
            raw_hour (str): The session interval.
            date (str): The session date.
            pen_price (str): The price for Iberian Peninsula and Balearic Islands.
            can_price (str): The price for Canary Island and Melilla.
        """
        self.__raw_hour = raw_hour
        self.__date = date
        self.__pen_price = float(pen_price.replace(",", "."))
        self.__can_price = float(can_price.replace(",", "."))
        splitted_hour = self.__raw_hour.split("-")
        self.left_hour = datetime.now().replace(
            hour=int(
                splitted_hour[0] if splitted_hour[0] != "24" else "00"
            ),
            minute=0,
            second=0,
        )
        self.right_hour = datetime.now().replace(
            hour=int(
                splitted_hour[1] if splitted_hour[1] != "24" else "00"
            ),
            minute=0,
            second=0,
        )
        self._cheap = False

    def get_raw_hour(self) -> str:
        """
        The function to get the raw session interval.

        Returns:
            str: The session interval.
        """
        return self.__raw_hour

    def get_pen_price(self) -> float:
        """
        The function to get the price for Iberian Peninsula and Balearic Islands.

        Returns:
            str: The price for Iberian Peninsula and Balearic Islands.
        """
        return self.__pen_price

    def get_can_price(self) -> float:
        """
        The function to get the price for Canary Island and Melilla.

        Returns:
            str: The price for Canary Island and Melilla.
        """
        return self.__can_price

    def is_cheap(self):
        """
        The function to get the cheap value

        Returns:
            bool: True if is cheap price False if not
        """
        return self._cheap

    def set_cheap(self, is_cheap:bool):
        """
        Setter method to cheap attribute
        Args:
            is_cheap (bool): True or False
        """
        self._cheap = is_cheap

    def __repr__(self) -> str:
        """
        The function to represent Price object.

        Returns:
            str: The Price object formatted as a string.
        """
        return "('raw-hour':'%s','date':'%s','pen-price':'%.2f','can-price':'%.2f')" % (
            self.__raw_hour,
            self.__date,
            self.get_pen_price(),
            self.get_can_price(),
        )

    def __str__(self) -> str:
        """
        The function to print Price object.

        Returns:
            str: The Price object formatted as a string.
        """
        return "('raw-hour':'%s','date':'%s','pen-price':'%.2f','can-price':'%.2f')" % (
            self.__raw_hour,
            self.__date,
            self.get_pen_price(),
            self.get_can_price(),
        )



# MIT License

# Copyright (c) 2021 Jorge Marín

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# Contributors:
#     Jorge Marín - initial Package version
"""
module utils
"""
import json
import requests

from requests import RequestException

class DocumentManager:
    """
    Class to manage data retrieval from REE API.
    """
    def __init__(self, api_token) -> None:
        """
        Constructor of the DocumentManager class.

        Args:
            api_token (str): Access token key for access to API.
        """
        self.__api_token = api_token

    def get_json_document(self,endpoint) -> dict:
        """
        The function to get json documents from REE API.

        Args:
            endpoint (str): The url of the endpoint.

        Returns:
            dict: A dictionary containing all session info.
        """
        return self.__download_json(endpoint)

    def reload_api_key(self, api_token):
        """
        The function to reload api token.
        Args:
            api_token (str): Access token key for access to API.
        """
        self.__api_token = api_token

    def __download_json(self, endpoint) -> dict:
        """
        The function to get json documents from REE API.

        Args:
            endpoint (str): The url of the endpoint

        Returns:
            dict: A dictionary containing all session info.
        """
        header = {
            "Accept": "application/json; application/vnd.esios-api-v1+json",
            "Content-Type": "application/json",
            "Host": "api.esios.ree.es",
            "Authorization": f"Token token={self.__api_token}",
        }
        try:
            response = requests.request("GET", endpoint, headers=header, verify=True)
            resp = json.loads(response.text)
            return resp
        except RequestException as re_exc:
            return re_exc



# MIT License

# Copyright (c) 2021 Jorge Marín

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

# Contributors:
#     Jorge Marín - initial Package version
"""
markets module
"""

from typing import Callable, List

from ree_connector.objects import Price
from ree_connector.utils import DocumentManager


class PvpcMarket:
    """
    This is a class for retrieve information of a determinated market session based on the date.

    Raises:
        ValueError: Throwed if value for location is not 'PEN' or 'CAN' case-insensitive

    Attributes:
        date (str): The market date you want to instantiate, format:'dd-mm-yyyy'.
        api_token (str): Authorized token to access REE-ESIOS API.
        location (str): Selector 'pen' or 'can' for work with prices for work with two locations.
    """

    def __init__(self, date:str, api_token:str, location:str) -> None:
        """
        The constructor for PvpcMarket class.

        Parameters:
           date (str): The market date you want to instantiate, format:'dd-mm-yyyy'.
           api_token (str): Authorized token to access REE-ESIOS API.
           location (str): Selector 'pen' or 'can' for work with two locations.
        """
        self.__pvpc_url = (
            "http://api.esios.ree.es/archives/70/download_json?locale=es&date="
        )

        self.__location = location.lower()

        if not self.__location in ("pen", "can"):
            raise ValueError(
                "Only 'PEN(Iberian peninsula and Balearic Islands),\
                CAN(Canary Islands and Melilla)' values are allowed"
            )

        if self.__location == "pen":
            self.__price_location_lambda: Callable[
                [Price], float
            ] = lambda x: x.get_pen_price()
        elif self.__location == "can":
            self.__price_location_lambda: Callable[
                [Price], float
            ] = lambda x: x.get_can_price()

        self.json_object = DocumentManager(api_token).get_json_document(f"{self.__pvpc_url}{date}")
        self.__prices = [
            Price(interval["Hora"], date, interval["PCB"], interval["CYM"])
            for interval in self.json_object["PVPC"]
        ]

    def get_n_cheapest_hours(self, n_hours:int) -> List[Price]:
        """
        The function to get the n cheapests prices of the market session.

        Parameters:
            n_hours (int): The number of cheapests intervals to retrieve.

        Returns:
            list[Price]: A list containing the n cheapests prices.
        """
        cheapests = self.__prices
        _empty = [cheap.set_cheap(False) for cheap in cheapests]
        cheapests.sort(key=self.__price_location_lambda)
        cheapests = cheapests[: n_hours if n_hours <= 24 else 24]
        _empty = [cheap.set_cheap(True) for cheap in cheapests]
        return cheapests

    def get_session_min_price(self) -> Price:
        """
        The function get the cheapest price of the session.

        Returns:
            Price: A Price object.
        """
        return min(self.__prices, key=self.__price_location_lambda)

    def get_session_max_price(self) -> Price:
        """
        The function get the most expensive price of the session.

        Returns:
            Price: A Price object.
        """
        return max(self.__prices, key=self.__price_location_lambda)

    def get_session_prices(self)-> List[Price]:
        """
        The function to get all session prices.

        Returns:
            list[Price]: A list containing all session prices.
        """
        return self.__prices

    def get_pen_data_plot(self) -> List[tuple]:
        """
        The function to get Iberian Peninsula and Balearic Islands
        price ordered by hour

        Returns:
            List[tuple]: A list of tuples (hour,price) ordered by hour
        """
        return [(price.get_raw_hour(),price.get_pen_price()) for price in self.__prices]

    def get_can_data_plot(self) -> List[tuple]:
        """
        Iberian Peninsula and Balearic Islands or Canary Islands and Melilla

        Returns:
            List[tuple]: A list of tuples (hour,price) ordered by hour
        """
        return [(price.get_raw_hour(),price.get_can_price()) for price in self.__prices]
